@@ -56,29 +56,19 @@ struct _GeeIteratorIface {
  */
 struct _GeeIterableIface {
 	GTypeInterface parent_iface;
-	GType (*get_element_type) (GeeIterable* self);
 	GeeIterator* (*iterator) (GeeIterable* self);
+	GType (*get_element_type) (GeeIterable* self);
 };
 
 
 
 GType gee_iterator_get_type (void);
 GType gee_iterable_get_type (void);
-GType gee_iterable_get_element_type (GeeIterable* self);
 GeeIterator* gee_iterable_iterator (GeeIterable* self);
+GType gee_iterable_get_element_type (GeeIterable* self);
 
 
 
-/**
- * Returns the type of the elements contained in this collection.
- *
- * @return the type of the elements contained in this collection
- */
-GType gee_iterable_get_element_type (GeeIterable* self) {
-	return GEE_ITERABLE_GET_INTERFACE (self)->get_element_type (self);
-}
-
-
 /**
  * Returns a Iterator that can be used for simple iteration over a
  * collection.
@@ -91,10 +81,16 @@ GeeIterator* gee_iterable_iterator (GeeIterable* self) {
 }
 
 
+GType gee_iterable_get_element_type (GeeIterable* self) {
+	return GEE_ITERABLE_GET_INTERFACE (self)->get_element_type (self);
+}
+
+
 static void gee_iterable_base_init (GeeIterableIface * iface) {
 	static gboolean initialized = FALSE;
 	if (!initialized) {
 		initialized = TRUE;
+		g_object_interface_install_property (iface, g_param_spec_gtype ("element-type", "element-type", "element-type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
 	}
 }
 
