@@ -86,8 +86,8 @@ struct _GeeIteratorIface {
  */
 struct _GeeIterableIface {
 	GTypeInterface parent_iface;
-	GType (*get_element_type) (GeeIterable* self);
 	GeeIterator* (*iterator) (GeeIterable* self);
+	GType (*get_element_type) (GeeIterable* self);
 };
 
 /**
@@ -100,7 +100,13 @@ struct _GeeCollectionIface {
 	gboolean (*add) (GeeCollection* self, gconstpointer item);
 	gboolean (*remove) (GeeCollection* self, gconstpointer item);
 	void (*clear) (GeeCollection* self);
+	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
+	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
 	gint (*get_size) (GeeCollection* self);
+	gboolean (*get_is_empty) (GeeCollection* self);
 };
 
 /**
@@ -152,16 +158,16 @@ GType gee_read_only_collection_get_type (void);
 enum  {
 	GEE_READ_ONLY_COLLECTION_DUMMY_PROPERTY,
 	GEE_READ_ONLY_COLLECTION_SIZE,
+	GEE_READ_ONLY_COLLECTION_IS_EMPTY,
 	GEE_READ_ONLY_COLLECTION_COLLECTION,
+	GEE_READ_ONLY_COLLECTION_ELEMENT_TYPE,
 	GEE_READ_ONLY_COLLECTION_G_TYPE,
 	GEE_READ_ONLY_COLLECTION_G_DUP_FUNC,
 	GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC
 };
-void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value);
+static void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value);
 GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
 GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
-GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
-static GType gee_read_only_collection_real_get_element_type (GeeIterable* base);
 static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static GType gee_read_only_collection_iterator_get_type (void);
@@ -172,7 +178,16 @@ static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gco
 static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item);
 static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item);
 static void gee_read_only_collection_real_clear (GeeCollection* base);
+static gboolean gee_read_only_collection_real_add_all (GeeCollection* base, GeeCollection* collection);
+gboolean gee_iterator_next (GeeIterator* self);
+gpointer gee_iterator_get (GeeIterator* self);
+static gboolean gee_read_only_collection_real_contains_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_collection_real_remove_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_collection_real_retain_all (GeeCollection* base, GeeCollection* collection);
+gpointer* gee_collection_to_array (GeeCollection* self, int* result_length1);
+static gpointer* gee_read_only_collection_real_to_array (GeeCollection* base, int* result_length1);
 gint gee_collection_get_size (GeeCollection* self);
+gboolean gee_collection_get_is_empty (GeeCollection* self);
 #define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_COLLECTION_ITERATOR_DUMMY_PROPERTY,
@@ -182,7 +197,6 @@ enum  {
 };
 static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base);
-static GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
 static void gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
 static void gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_read_only_collection_finalize (GObject* obj);
@@ -195,7 +209,7 @@ GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GT
 	GParameter * __params;
 	GParameter * __params_it;
 	GeeReadOnlyCollection * self;
-	__params = g_new0 (GParameter, 3);
+	__params = g_new0 (GParameter, 4);
 	__params_it = __params;
 	__params_it->name = "g-type";
 	g_value_init (&__params_it->value, G_TYPE_GTYPE);
@@ -209,8 +223,11 @@ GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GT
 	g_value_init (&__params_it->value, G_TYPE_POINTER);
 	g_value_set_pointer (&__params_it->value, g_destroy_func);
 	__params_it++;
+	__params_it->name = "collection";
+	g_value_init (&__params_it->value, GEE_TYPE_COLLECTION);
+	g_value_set_object (&__params_it->value, collection);
+	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
-	gee_read_only_collection_set_collection (self, collection);
 	while (__params_it > __params) {
 		--__params_it;
 		g_value_unset (&__params_it->value);
@@ -225,15 +242,6 @@ GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFun
 }
 
 
-static GType gee_read_only_collection_real_get_element_type (GeeIterable* base) {
-	GeeReadOnlyCollection * self;
-	GType result;
-	self = (GeeReadOnlyCollection*) base;
-	result = self->priv->g_type;
-	return result;
-}
-
-
 static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base) {
 	GeeReadOnlyCollection * self;
 	GeeIterator* result;
@@ -283,6 +291,74 @@ static void gee_read_only_collection_real_clear (GeeCollection* base) {
 }
 
 
+static gboolean gee_read_only_collection_real_add_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyCollection * self;
+	gboolean result;
+	self = (GeeReadOnlyCollection*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gboolean gee_read_only_collection_real_contains_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyCollection * self;
+	gboolean result;
+	self = (GeeReadOnlyCollection*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	{
+		GeeIterator* _element_it;
+		_element_it = gee_iterable_iterator ((GeeIterable*) collection);
+		while (TRUE) {
+			gpointer element;
+			if (!gee_iterator_next (_element_it)) {
+				break;
+			}
+			element = gee_iterator_get (_element_it);
+			if (!gee_collection_contains ((GeeCollection*) self, element)) {
+				result = FALSE;
+				((element == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (element = (self->priv->g_destroy_func (element), NULL));
+				(_element_it == NULL) ? NULL : (_element_it = (g_object_unref (_element_it), NULL));
+				return result;
+			}
+			((element == NULL) || (self->priv->g_destroy_func == NULL)) ? NULL : (element = (self->priv->g_destroy_func (element), NULL));
+		}
+		(_element_it == NULL) ? NULL : (_element_it = (g_object_unref (_element_it), NULL));
+	}
+	result = TRUE;
+	return result;
+}
+
+
+static gboolean gee_read_only_collection_real_remove_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyCollection * self;
+	gboolean result;
+	self = (GeeReadOnlyCollection*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gboolean gee_read_only_collection_real_retain_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyCollection * self;
+	gboolean result;
+	self = (GeeReadOnlyCollection*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gpointer* gee_read_only_collection_real_to_array (GeeCollection* base, int* result_length1) {
+	GeeReadOnlyCollection * self;
+	gpointer* result;
+	gint _tmp0_;
+	gpointer* _tmp1_;
+	self = (GeeReadOnlyCollection*) base;
+	_tmp1_ = NULL;
+	result = (_tmp1_ = gee_collection_to_array (self->priv->_collection, &_tmp0_), *result_length1 = _tmp0_, _tmp1_);
+	return result;
+}
+
+
 static gint gee_read_only_collection_real_get_size (GeeCollection* base) {
 	gint result;
 	GeeReadOnlyCollection* self;
@@ -292,7 +368,16 @@ static gint gee_read_only_collection_real_get_size (GeeCollection* base) {
 }
 
 
-void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value) {
+static gboolean gee_read_only_collection_real_get_is_empty (GeeCollection* base) {
+	gboolean result;
+	GeeReadOnlyCollection* self;
+	self = (GeeReadOnlyCollection*) base;
+	result = gee_collection_get_is_empty (self->priv->_collection);
+	return result;
+}
+
+
+static void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCollection* value) {
 	GeeCollection* _tmp1_;
 	GeeCollection* _tmp0_;
 	g_return_if_fail (self != NULL);
@@ -303,6 +388,15 @@ void gee_read_only_collection_set_collection (GeeReadOnlyCollection* self, GeeCo
 }
 
 
+static GType gee_read_only_collection_real_get_element_type (GeeIterable* base) {
+	GType result;
+	GeeReadOnlyCollection* self;
+	self = (GeeReadOnlyCollection*) base;
+	result = self->priv->g_type;
+	return result;
+}
+
+
 static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlyCollectionIterator * self;
 	gboolean result;
@@ -431,14 +525,16 @@ static void gee_read_only_collection_class_init (GeeReadOnlyCollectionClass * kl
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_SIZE, "size");
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_COLLECTION, g_param_spec_object ("collection", "collection", "collection", GEE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_IS_EMPTY, "is-empty");
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_COLLECTION, g_param_spec_object ("collection", "collection", "collection", GEE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ELEMENT_TYPE, "element-type");
 }
 
 
 static void gee_read_only_collection_gee_iterable_interface_init (GeeIterableIface * iface) {
 	gee_read_only_collection_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
-	iface->get_element_type = gee_read_only_collection_real_get_element_type;
 	iface->iterator = gee_read_only_collection_real_iterator;
+	iface->get_element_type = gee_read_only_collection_real_get_element_type;
 }
 
 
@@ -448,7 +544,13 @@ static void gee_read_only_collection_gee_collection_interface_init (GeeCollectio
 	iface->add = gee_read_only_collection_real_add;
 	iface->remove = gee_read_only_collection_real_remove;
 	iface->clear = gee_read_only_collection_real_clear;
+	iface->add_all = gee_read_only_collection_real_add_all;
+	iface->contains_all = gee_read_only_collection_real_contains_all;
+	iface->remove_all = gee_read_only_collection_real_remove_all;
+	iface->retain_all = gee_read_only_collection_real_retain_all;
+	iface->to_array = gee_read_only_collection_real_to_array;
 	iface->get_size = gee_read_only_collection_real_get_size;
+	iface->get_is_empty = gee_read_only_collection_real_get_is_empty;
 }
 
 
@@ -487,6 +589,12 @@ static void gee_read_only_collection_get_property (GObject * object, guint prope
 		case GEE_READ_ONLY_COLLECTION_SIZE:
 		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
 		break;
+		case GEE_READ_ONLY_COLLECTION_IS_EMPTY:
+		g_value_set_boolean (value, gee_collection_get_is_empty ((GeeCollection*) self));
+		break;
+		case GEE_READ_ONLY_COLLECTION_ELEMENT_TYPE:
+		g_value_set_gtype (value, gee_iterable_get_element_type ((GeeIterable*) self));
+		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
