@@ -20,9 +20,53 @@
  * 	Jürg Billeter <j@bitron.ch>
  */
 
-#include <gee/readonlylist.h>
+#include <glib.h>
+#include <glib-object.h>
 
 
+#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
+#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
+#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
+#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))
+
+typedef struct _GeeIterable GeeIterable;
+typedef struct _GeeIterableIface GeeIterableIface;
+
+#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
+#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
+#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
+#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))
+
+typedef struct _GeeIterator GeeIterator;
+typedef struct _GeeIteratorIface GeeIteratorIface;
+
+#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
+#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
+#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
+#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))
+
+typedef struct _GeeCollection GeeCollection;
+typedef struct _GeeCollectionIface GeeCollectionIface;
+
+#define GEE_TYPE_LIST (gee_list_get_type ())
+#define GEE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LIST, GeeList))
+#define GEE_IS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LIST))
+#define GEE_LIST_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_LIST, GeeListIface))
+
+typedef struct _GeeList GeeList;
+typedef struct _GeeListIface GeeListIface;
+
+#define GEE_TYPE_READ_ONLY_LIST (gee_read_only_list_get_type ())
+#define GEE_READ_ONLY_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyList))
+#define GEE_READ_ONLY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListClass))
+#define GEE_IS_READ_ONLY_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_LIST))
+#define GEE_IS_READ_ONLY_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_LIST))
+#define GEE_READ_ONLY_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListClass))
+
+typedef struct _GeeReadOnlyList GeeReadOnlyList;
+typedef struct _GeeReadOnlyListClass GeeReadOnlyListClass;
+typedef struct _GeeReadOnlyListPrivate GeeReadOnlyListPrivate;
+
 #define GEE_READ_ONLY_LIST_TYPE_ITERATOR (gee_read_only_list_iterator_get_type ())
 #define GEE_READ_ONLY_LIST_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIterator))
 #define GEE_READ_ONLY_LIST_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIteratorClass))
@@ -34,6 +78,80 @@ typedef struct _GeeReadOnlyListIterator GeeReadOnlyListIterator;
 typedef struct _GeeReadOnlyListIteratorClass GeeReadOnlyListIteratorClass;
 typedef struct _GeeReadOnlyListIteratorPrivate GeeReadOnlyListIteratorPrivate;
 
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIteratorIface {
+	GTypeInterface parent_iface;
+	gboolean (*next) (GeeIterator* self);
+	gpointer (*get) (GeeIterator* self);
+};
+
+/**
+ * Implemented by classes that support a simple iteration over instances of the
+ * collection.
+ */
+struct _GeeIterableIface {
+	GTypeInterface parent_iface;
+	GeeIterator* (*iterator) (GeeIterable* self);
+	GType (*get_element_type) (GeeIterable* self);
+};
+
+/**
+ * Serves as the base interface for implementing collection classes. Defines
+ * size, iteration, and modification methods.
+ */
+struct _GeeCollectionIface {
+	GTypeInterface parent_iface;
+	gboolean (*contains) (GeeCollection* self, gconstpointer item);
+	gboolean (*add) (GeeCollection* self, gconstpointer item);
+	gboolean (*remove) (GeeCollection* self, gconstpointer item);
+	void (*clear) (GeeCollection* self);
+	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
+	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
+	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
+	gint (*get_size) (GeeCollection* self);
+	gboolean (*get_is_empty) (GeeCollection* self);
+};
+
+/**
+ * Represents a collection of items in a well-defined order.
+ */
+struct _GeeListIface {
+	GTypeInterface parent_iface;
+	gpointer (*get) (GeeList* self, gint index);
+	void (*set) (GeeList* self, gint index, gconstpointer item);
+	gint (*index_of) (GeeList* self, gconstpointer item);
+	void (*insert) (GeeList* self, gint index, gconstpointer item);
+	void (*remove_at) (GeeList* self, gint index);
+	GeeList* (*slice) (GeeList* self, gint start, gint stop);
+	gpointer (*first) (GeeList* self);
+	gpointer (*last) (GeeList* self);
+	void (*insert_all) (GeeList* self, gint index, GeeCollection* collection);
+};
+
+/**
+ * Represents a read-only collection of items in a well-defined order.
+ */
+struct _GeeReadOnlyList {
+	GObject parent_instance;
+	GeeReadOnlyListPrivate * priv;
+};
+
+struct _GeeReadOnlyListClass {
+	GObjectClass parent_class;
+};
+
+struct _GeeReadOnlyListPrivate {
+	GType g_type;
+	GBoxedCopyFunc g_dup_func;
+	GDestroyNotify g_destroy_func;
+	GeeList* _list;
+};
+
 struct _GeeReadOnlyListIterator {
 	GObject parent_instance;
 	GeeReadOnlyListIteratorPrivate * priv;
@@ -43,42 +161,71 @@ struct _GeeReadOnlyListIteratorClass {
 	GObjectClass parent_class;
 };
 
-
-
-struct _GeeReadOnlyListPrivate {
-	GeeList* _list;
-	GType g_type;
-	GBoxedCopyFunc g_dup_func;
-	GDestroyNotify g_destroy_func;
-};
-
-#define GEE_READ_ONLY_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListPrivate))
-enum  {
-	GEE_READ_ONLY_LIST_DUMMY_PROPERTY,
-	GEE_READ_ONLY_LIST_SIZE,
-	GEE_READ_ONLY_LIST_LIST,
-	GEE_READ_ONLY_LIST_G_TYPE,
-	GEE_READ_ONLY_LIST_G_DUP_FUNC,
-	GEE_READ_ONLY_LIST_G_DESTROY_FUNC
-};
-static GType gee_read_only_list_real_get_element_type (GeeIterable* base);
-static GeeIterator* gee_read_only_list_real_iterator (GeeIterable* base);
-static gboolean gee_read_only_list_real_contains (GeeCollection* base, gconstpointer item);
-static gint gee_read_only_list_real_index_of (GeeList* base, gconstpointer item);
-static gboolean gee_read_only_list_real_add (GeeCollection* base, gconstpointer item);
-static gboolean gee_read_only_list_real_remove (GeeCollection* base, gconstpointer item);
-static void gee_read_only_list_real_insert (GeeList* base, gint index, gconstpointer item);
-static void gee_read_only_list_real_remove_at (GeeList* base, gint index);
-static gpointer gee_read_only_list_real_get (GeeList* base, gint index);
-static void gee_read_only_list_real_set (GeeList* base, gint index, gconstpointer o);
-static void gee_read_only_list_real_clear (GeeCollection* base);
-static GeeList* gee_read_only_list_real_slice (GeeList* base, gint start, gint stop);
 struct _GeeReadOnlyListIteratorPrivate {
 	GType g_type;
 	GBoxedCopyFunc g_dup_func;
 	GDestroyNotify g_destroy_func;
 };
 
+
+static gpointer gee_read_only_list_iterator_parent_class = NULL;
+static GeeIteratorIface* gee_read_only_list_iterator_gee_iterator_parent_iface = NULL;
+static gpointer gee_read_only_list_parent_class = NULL;
+static GeeIterableIface* gee_read_only_list_gee_iterable_parent_iface = NULL;
+static GeeCollectionIface* gee_read_only_list_gee_collection_parent_iface = NULL;
+static GeeListIface* gee_read_only_list_gee_list_parent_iface = NULL;
+
+GType gee_iterator_get_type (void);
+GType gee_iterable_get_type (void);
+GType gee_collection_get_type (void);
+GType gee_list_get_type (void);
+GType gee_read_only_list_get_type (void);
+#define GEE_READ_ONLY_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_LIST, GeeReadOnlyListPrivate))
+enum  {
+	GEE_READ_ONLY_LIST_DUMMY_PROPERTY,
+	GEE_READ_ONLY_LIST_SIZE,
+	GEE_READ_ONLY_LIST_IS_EMPTY,
+	GEE_READ_ONLY_LIST_LIST,
+	GEE_READ_ONLY_LIST_ELEMENT_TYPE,
+	GEE_READ_ONLY_LIST_G_TYPE,
+	GEE_READ_ONLY_LIST_G_DUP_FUNC,
+	GEE_READ_ONLY_LIST_G_DESTROY_FUNC
+};
+static void gee_read_only_list_set_list (GeeReadOnlyList* self, GeeList* value);
+GeeReadOnlyList* gee_read_only_list_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeList* list);
+GeeReadOnlyList* gee_read_only_list_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeList* list);
+static GeeReadOnlyListIterator* gee_read_only_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GeeReadOnlyListIterator* gee_read_only_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
+static GType gee_read_only_list_iterator_get_type (void);
+GeeIterator* gee_iterable_iterator (GeeIterable* self);
+static GeeIterator* gee_read_only_list_real_iterator (GeeIterable* base);
+gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
+static gboolean gee_read_only_list_real_contains (GeeCollection* base, gconstpointer item);
+gint gee_list_index_of (GeeList* self, gconstpointer item);
+static gint gee_read_only_list_real_index_of (GeeList* base, gconstpointer item);
+static gboolean gee_read_only_list_real_add (GeeCollection* base, gconstpointer item);
+static gboolean gee_read_only_list_real_remove (GeeCollection* base, gconstpointer item);
+static void gee_read_only_list_real_insert (GeeList* base, gint index, gconstpointer item);
+static void gee_read_only_list_real_remove_at (GeeList* base, gint index);
+gpointer gee_list_get (GeeList* self, gint index);
+static gpointer gee_read_only_list_real_get (GeeList* base, gint index);
+static void gee_read_only_list_real_set (GeeList* base, gint index, gconstpointer o);
+static void gee_read_only_list_real_clear (GeeCollection* base);
+static GeeList* gee_read_only_list_real_slice (GeeList* base, gint start, gint stop);
+static gboolean gee_read_only_list_real_add_all (GeeCollection* base, GeeCollection* collection);
+gboolean gee_collection_contains_all (GeeCollection* self, GeeCollection* collection);
+static gboolean gee_read_only_list_real_contains_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_list_real_remove_all (GeeCollection* base, GeeCollection* collection);
+static gboolean gee_read_only_list_real_retain_all (GeeCollection* base, GeeCollection* collection);
+gpointer gee_list_first (GeeList* self);
+static gpointer gee_read_only_list_real_first (GeeList* base);
+gpointer gee_list_last (GeeList* self);
+static gpointer gee_read_only_list_real_last (GeeList* base);
+static void gee_read_only_list_real_insert_all (GeeList* base, gint index, GeeCollection* collection);
+gpointer* gee_collection_to_array (GeeCollection* self, int* result_length1);
+static gpointer* gee_read_only_list_real_to_array (GeeCollection* base, int* result_length1);
+gint gee_collection_get_size (GeeCollection* self);
+gboolean gee_collection_get_is_empty (GeeCollection* self);
 #define GEE_READ_ONLY_LIST_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_LIST_TYPE_ITERATOR, GeeReadOnlyListIteratorPrivate))
 enum  {
 	GEE_READ_ONLY_LIST_ITERATOR_DUMMY_PROPERTY,
@@ -88,16 +235,11 @@ enum  {
 };
 static gboolean gee_read_only_list_iterator_real_next (GeeIterator* base);
 static gpointer gee_read_only_list_iterator_real_get (GeeIterator* base);
-static GeeReadOnlyListIterator* gee_read_only_list_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static GeeReadOnlyListIterator* gee_read_only_list_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
-static gpointer gee_read_only_list_iterator_parent_class = NULL;
-static GeeIteratorIface* gee_read_only_list_iterator_gee_iterator_parent_iface = NULL;
-static GType gee_read_only_list_iterator_get_type (void);
-static gpointer gee_read_only_list_parent_class = NULL;
-static GeeIterableIface* gee_read_only_list_gee_iterable_parent_iface = NULL;
-static GeeCollectionIface* gee_read_only_list_gee_collection_parent_iface = NULL;
-static GeeListIface* gee_read_only_list_gee_list_parent_iface = NULL;
+static void gee_read_only_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 static void gee_read_only_list_finalize (GObject* obj);
+static void gee_read_only_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
+static void gee_read_only_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
 
 
 
@@ -105,7 +247,7 @@ GeeReadOnlyList* gee_read_only_list_construct (GType object_type, GType g_type,
 	GParameter * __params;
 	GParameter * __params_it;
 	GeeReadOnlyList * self;
-	__params = g_new0 (GParameter, 3);
+	__params = g_new0 (GParameter, 4);
 	__params_it = __params;
 	__params_it->name = "g-type";
 	g_value_init (&__params_it->value, G_TYPE_GTYPE);
@@ -119,8 +261,11 @@ GeeReadOnlyList* gee_read_only_list_construct (GType object_type, GType g_type,
 	g_value_init (&__params_it->value, G_TYPE_POINTER);
 	g_value_set_pointer (&__params_it->value, g_destroy_func);
 	__params_it++;
+	__params_it->name = "list";
+	g_value_init (&__params_it->value, GEE_TYPE_LIST);
+	g_value_set_object (&__params_it->value, list);
+	__params_it++;
 	self = g_object_newv (object_type, __params_it - __params, __params);
-	gee_read_only_list_set_list (self, list);
 	while (__params_it > __params) {
 		--__params_it;
 		g_value_unset (&__params_it->value);
@@ -135,45 +280,48 @@ GeeReadOnlyList* gee_read_only_list_new (GType g_type, GBoxedCopyFunc g_dup_func
 }
 
 
-static GType gee_read_only_list_real_get_element_type (GeeIterable* base) {
-	GeeReadOnlyList * self;
-	self = (GeeReadOnlyList*) base;
-	return self->priv->g_type;
-}
-
-
 static GeeIterator* gee_read_only_list_real_iterator (GeeIterable* base) {
 	GeeReadOnlyList * self;
+	GeeIterator* result;
 	self = (GeeReadOnlyList*) base;
 	if (self->priv->_list == NULL) {
-		return (GeeIterator*) gee_read_only_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		result = (GeeIterator*) gee_read_only_list_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func);
+		return result;
 	}
-	return gee_iterable_iterator ((GeeIterable*) self->priv->_list);
+	result = gee_iterable_iterator ((GeeIterable*) self->priv->_list);
+	return result;
 }
 
 
 static gboolean gee_read_only_list_real_contains (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyList * self;
+	gboolean result;
 	self = (GeeReadOnlyList*) base;
 	if (self->priv->_list == NULL) {
-		return FALSE;
+		result = FALSE;
+		return result;
 	}
-	return gee_collection_contains ((GeeCollection*) self->priv->_list, item);
+	result = gee_collection_contains ((GeeCollection*) self->priv->_list, item);
+	return result;
 }
 
 
 static gint gee_read_only_list_real_index_of (GeeList* base, gconstpointer item) {
 	GeeReadOnlyList * self;
+	gint result;
 	self = (GeeReadOnlyList*) base;
 	if (self->priv->_list == NULL) {
-		return -1;
+		result = -1;
+		return result;
 	}
-	return gee_list_index_of (self->priv->_list, item);
+	result = gee_list_index_of (self->priv->_list, item);
+	return result;
 }
 
 
 static gboolean gee_read_only_list_real_add (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyList * self;
+	gboolean result;
 	self = (GeeReadOnlyList*) base;
 	g_assert_not_reached ();
 }
@@ -181,6 +329,7 @@ static gboolean gee_read_only_list_real_add (GeeCollection* base, gconstpointer
 
 static gboolean gee_read_only_list_real_remove (GeeCollection* base, gconstpointer item) {
 	GeeReadOnlyList * self;
+	gboolean result;
 	self = (GeeReadOnlyList*) base;
 	g_assert_not_reached ();
 }
@@ -202,11 +351,14 @@ static void gee_read_only_list_real_remove_at (GeeList* base, gint index) {
 
 static gpointer gee_read_only_list_real_get (GeeList* base, gint index) {
 	GeeReadOnlyList * self;
+	gpointer result;
 	self = (GeeReadOnlyList*) base;
 	if (self->priv->_list == NULL) {
-		return NULL;
+		result = NULL;
+		return result;
 	}
-	return gee_list_get (self->priv->_list, index);
+	result = gee_list_get (self->priv->_list, index);
+	return result;
 }
 
 
@@ -226,40 +378,152 @@ static void gee_read_only_list_real_clear (GeeCollection* base) {
 
 static GeeList* gee_read_only_list_real_slice (GeeList* base, gint start, gint stop) {
 	GeeReadOnlyList * self;
+	GeeList* result;
 	self = (GeeReadOnlyList*) base;
 	g_assert_not_reached ();
 }
 
 
-static gint gee_read_only_list_real_get_size (GeeCollection* base) {
-	GeeReadOnlyList* self;
+static gboolean gee_read_only_list_real_add_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyList * self;
+	gboolean result;
 	self = (GeeReadOnlyList*) base;
-	return gee_collection_get_size ((GeeCollection*) self->priv->_list);
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
 }
 
 
-void gee_read_only_list_set_list (GeeReadOnlyList* self, GeeList* value) {
-	GeeList* _tmp1;
-	GeeList* _tmp0;
+static gboolean gee_read_only_list_real_contains_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyList * self;
+	gboolean result;
+	self = (GeeReadOnlyList*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	if (self->priv->_list == NULL) {
+		result = FALSE;
+		return result;
+	}
+	result = gee_collection_contains_all ((GeeCollection*) self->priv->_list, collection);
+	return result;
+}
+
+
+static gboolean gee_read_only_list_real_remove_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyList * self;
+	gboolean result;
+	self = (GeeReadOnlyList*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gboolean gee_read_only_list_real_retain_all (GeeCollection* base, GeeCollection* collection) {
+	GeeReadOnlyList * self;
+	gboolean result;
+	self = (GeeReadOnlyList*) base;
+	g_return_val_if_fail (collection != NULL, FALSE);
+	g_assert_not_reached ();
+}
+
+
+static gpointer gee_read_only_list_real_first (GeeList* base) {
+	GeeReadOnlyList * self;
+	gpointer result;
+	self = (GeeReadOnlyList*) base;
+	if (self->priv->_list == NULL) {
+		result = NULL;
+		return result;
+	}
+	result = gee_list_first (self->priv->_list);
+	return result;
+}
+
+
+static gpointer gee_read_only_list_real_last (GeeList* base) {
+	GeeReadOnlyList * self;
+	gpointer result;
+	self = (GeeReadOnlyList*) base;
+	if (self->priv->_list == NULL) {
+		result = NULL;
+		return result;
+	}
+	result = gee_list_last (self->priv->_list);
+	return result;
+}
+
+
+static void gee_read_only_list_real_insert_all (GeeList* base, gint index, GeeCollection* collection) {
+	GeeReadOnlyList * self;
+	self = (GeeReadOnlyList*) base;
+	g_return_if_fail (collection != NULL);
+	g_assert_not_reached ();
+}
+
+
+static gpointer* gee_read_only_list_real_to_array (GeeCollection* base, int* result_length1) {
+	GeeReadOnlyList * self;
+	gpointer* result;
+	gint _tmp0_;
+	gpointer* _tmp1_;
+	self = (GeeReadOnlyList*) base;
+	_tmp1_ = NULL;
+	result = (_tmp1_ = gee_collection_to_array ((GeeCollection*) self->priv->_list, &_tmp0_), *result_length1 = _tmp0_, _tmp1_);
+	return result;
+}
+
+
+static gint gee_read_only_list_real_get_size (GeeCollection* base) {
+	gint result;
+	GeeReadOnlyList* self;
+	self = (GeeReadOnlyList*) base;
+	result = gee_collection_get_size ((GeeCollection*) self->priv->_list);
+	return result;
+}
+
+
+static gboolean gee_read_only_list_real_get_is_empty (GeeCollection* base) {
+	gboolean result;
+	GeeReadOnlyList* self;
+	self = (GeeReadOnlyList*) base;
+	result = gee_collection_get_is_empty ((GeeCollection*) self->priv->_list);
+	return result;
+}
+
+
+static void gee_read_only_list_set_list (GeeReadOnlyList* self, GeeList* value) {
+	GeeList* _tmp1_;
+	GeeList* _tmp0_;
 	g_return_if_fail (self != NULL);
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	self->priv->_list = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)), _tmp1);
+	_tmp1_ = NULL;
+	_tmp0_ = NULL;
+	self->priv->_list = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_list == NULL) ? NULL : (self->priv->_list = (g_object_unref (self->priv->_list), NULL)), _tmp1_);
 	g_object_notify ((GObject *) self, "list");
 }
 
 
+static GType gee_read_only_list_real_get_element_type (GeeIterable* base) {
+	GType result;
+	GeeReadOnlyList* self;
+	self = (GeeReadOnlyList*) base;
+	result = self->priv->g_type;
+	return result;
+}
+
+
 static gboolean gee_read_only_list_iterator_real_next (GeeIterator* base) {
 	GeeReadOnlyListIterator * self;
+	gboolean result;
 	self = (GeeReadOnlyListIterator*) base;
-	return FALSE;
+	result = FALSE;
+	return result;
 }
 
 
 static gpointer gee_read_only_list_iterator_real_get (GeeIterator* base) {
 	GeeReadOnlyListIterator * self;
+	gpointer result;
 	self = (GeeReadOnlyListIterator*) base;
-	return NULL;
+	result = NULL;
+	return result;
 }
 
 
@@ -296,38 +560,6 @@ static GeeReadOnlyListIterator* gee_read_only_list_iterator_new (GType g_type, G
 }
 
 
-static void gee_read_only_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyListIterator * self;
-	gpointer boxed;
-	self = GEE_READ_ONLY_LIST_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-	}
-}
-
-
-static void gee_read_only_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyListIterator * self;
-	self = GEE_READ_ONLY_LIST_ITERATOR (object);
-	switch (property_id) {
-		default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-		break;
-		case GEE_READ_ONLY_LIST_ITERATOR_G_TYPE:
-		self->priv->g_type = g_value_get_gtype (value);
-		break;
-		case GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC:
-		self->priv->g_dup_func = g_value_get_pointer (value);
-		break;
-		case GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC:
-		self->priv->g_destroy_func = g_value_get_pointer (value);
-		break;
-	}
-}
-
-
 static void gee_read_only_list_iterator_class_init (GeeReadOnlyListIteratorClass * klass) {
 	gee_read_only_list_iterator_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GeeReadOnlyListIteratorPrivate));
@@ -363,14 +595,11 @@ static GType gee_read_only_list_iterator_get_type (void) {
 }
 
 
-static void gee_read_only_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyList * self;
+static void gee_read_only_list_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyListIterator * self;
 	gpointer boxed;
-	self = GEE_READ_ONLY_LIST (object);
+	self = GEE_READ_ONLY_LIST_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_LIST_SIZE:
-		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -378,23 +607,20 @@ static void gee_read_only_list_get_property (GObject * object, guint property_id
 }
 
 
-static void gee_read_only_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
-	GeeReadOnlyList * self;
-	self = GEE_READ_ONLY_LIST (object);
+static void gee_read_only_list_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyListIterator * self;
+	self = GEE_READ_ONLY_LIST_ITERATOR (object);
 	switch (property_id) {
-		case GEE_READ_ONLY_LIST_LIST:
-		gee_read_only_list_set_list (self, g_value_get_object (value));
-		break;
 		default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
-		case GEE_READ_ONLY_LIST_G_TYPE:
+		case GEE_READ_ONLY_LIST_ITERATOR_G_TYPE:
 		self->priv->g_type = g_value_get_gtype (value);
 		break;
-		case GEE_READ_ONLY_LIST_G_DUP_FUNC:
+		case GEE_READ_ONLY_LIST_ITERATOR_G_DUP_FUNC:
 		self->priv->g_dup_func = g_value_get_pointer (value);
 		break;
-		case GEE_READ_ONLY_LIST_G_DESTROY_FUNC:
+		case GEE_READ_ONLY_LIST_ITERATOR_G_DESTROY_FUNC:
 		self->priv->g_destroy_func = g_value_get_pointer (value);
 		break;
 	}
@@ -411,14 +637,16 @@ static void gee_read_only_list_class_init (GeeReadOnlyListClass * klass) {
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_SIZE, "size");
-	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_LIST, g_param_spec_object ("list", "list", "list", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_IS_EMPTY, "is-empty");
+	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_LIST, g_param_spec_object ("list", "list", "list", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_LIST_ELEMENT_TYPE, "element-type");
 }
 
 
 static void gee_read_only_list_gee_iterable_interface_init (GeeIterableIface * iface) {
 	gee_read_only_list_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
-	iface->get_element_type = gee_read_only_list_real_get_element_type;
 	iface->iterator = gee_read_only_list_real_iterator;
+	iface->get_element_type = gee_read_only_list_real_get_element_type;
 }
 
 
@@ -428,7 +656,13 @@ static void gee_read_only_list_gee_collection_interface_init (GeeCollectionIface
 	iface->add = gee_read_only_list_real_add;
 	iface->remove = gee_read_only_list_real_remove;
 	iface->clear = gee_read_only_list_real_clear;
+	iface->add_all = gee_read_only_list_real_add_all;
+	iface->contains_all = gee_read_only_list_real_contains_all;
+	iface->remove_all = gee_read_only_list_real_remove_all;
+	iface->retain_all = gee_read_only_list_real_retain_all;
+	iface->to_array = gee_read_only_list_real_to_array;
 	iface->get_size = gee_read_only_list_real_get_size;
+	iface->get_is_empty = gee_read_only_list_real_get_is_empty;
 }
 
 
@@ -440,6 +674,9 @@ static void gee_read_only_list_gee_list_interface_init (GeeListIface * iface) {
 	iface->get = gee_read_only_list_real_get;
 	iface->set = gee_read_only_list_real_set;
 	iface->slice = gee_read_only_list_real_slice;
+	iface->first = gee_read_only_list_real_first;
+	iface->last = gee_read_only_list_real_last;
+	iface->insert_all = gee_read_only_list_real_insert_all;
 }
 
 
@@ -472,5 +709,49 @@ GType gee_read_only_list_get_type (void) {
 }
 
 
+static void gee_read_only_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyList * self;
+	gpointer boxed;
+	self = GEE_READ_ONLY_LIST (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_LIST_SIZE:
+		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
+		break;
+		case GEE_READ_ONLY_LIST_IS_EMPTY:
+		g_value_set_boolean (value, gee_collection_get_is_empty ((GeeCollection*) self));
+		break;
+		case GEE_READ_ONLY_LIST_ELEMENT_TYPE:
+		g_value_set_gtype (value, gee_iterable_get_element_type ((GeeIterable*) self));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
+static void gee_read_only_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
+	GeeReadOnlyList * self;
+	self = GEE_READ_ONLY_LIST (object);
+	switch (property_id) {
+		case GEE_READ_ONLY_LIST_LIST:
+		gee_read_only_list_set_list (self, g_value_get_object (value));
+		break;
+		default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+		case GEE_READ_ONLY_LIST_G_TYPE:
+		self->priv->g_type = g_value_get_gtype (value);
+		break;
+		case GEE_READ_ONLY_LIST_G_DUP_FUNC:
+		self->priv->g_dup_func = g_value_get_pointer (value);
+		break;
+		case GEE_READ_ONLY_LIST_G_DESTROY_FUNC:
+		self->priv->g_destroy_func = g_value_get_pointer (value);
+		break;
+	}
+}
+
+
 
 
